"""
Python Script to take the chain MCMC samples and plot the margial posteriors
of each retrieval parameter. In this case of the Proterozoic Earth paper,
each retrieval was done at a given atmospheric composition (High, Med., and Low)
and each was modeled at 3 different SNRs (20, 30, and 50 for reflected light
and 10, 20, 30 ppm for transit retrievals). Additionally, each simulation in this
grid was run 10 times (ie 10 MCMC chains were generated) with random error bars
and the last 10K steps from each run were stitched together to formulate the
posteriors we are plotting here
"""

import numpy as np
from matplotlib import pyplot as plt
import h5py


#Retrieved Params for rfast v2.6.2 reflected light retrievals with Temperature
# But without N2
names  = [r"$\log f_{\rm O2}$",r"$\log f_{\rm H2O}$",\
r"$\log f_{\rm CO2}$",r"$\log f_{\rm O3}$",r"$\log f_{\rm CH4}$",\
r"$\log p_{0}$",r"$T_{0}$",r"log $A_{\rm s}$",r"$\log R_{\rm p}$",\
r"$\log M_{\rm p}$",r"$\log \Delta p_{\rm c}$",r"$\log p_{\rm t}$",\
r"$\log \tau_{\rm c}$",r"$\log f_{\rm c}$"]


#~*~*~*~**~**~*~*~*~**~*~*~*~*~**~*~*~*~*~*~*~*~*~*~**~*~**~~**~*~*~*~**~*~*~*~
#Reading in input data from the Simulation Grid

#Reflected High 20 SNR -------
samples1 = np.genfromtxt('PE_Refl_Temp_Retrieval_HighProfile_SNR20_ChainSample_2.txt',\
dtype=float)
#30 SNR
samples2 = np.genfromtxt('PE_Refl_Temp_Retrieval_HighProfile_SNR30_ChainSample_2.txt',\
dtype=float)
#50 SNR
samples3 = np.genfromtxt('PE_Refl_Temp_Retrieval_HighProfile_SNR50_ChainSample_2.txt',\
dtype=float)
#Reflected Avg 20 SNR --------
samples4 = np.genfromtxt('PE_Refl_Temp_Retrieval_AvgProfile_SNR20_ChainSample_2.txt',\
dtype=float)
#30 SNR
samples5 = np.genfromtxt('PE_Refl_Temp_Retrieval_AvgProfile_SNR30_ChainSample_2.txt',\
dtype=float)
#50 SNR
samples6 = np.genfromtxt('PE_Refl_Temp_Retrieval_AvgProfile_SNR50_ChainSample_2.txt',\
dtype=float)
#Reflected Low 20 SNR --------
samples7 = np.genfromtxt('PE_Refl_Temp_Retrieval_LowProfile_SNR20_ChainSample_2.txt',\
dtype=float)
#30 SNR
samples8 = np.genfromtxt('PE_Refl_Temp_Retrieval_LowProfile_SNR30_ChainSample_2.txt',\
dtype=float)
#50 SNR
samples9 = np.genfromtxt('PE_Refl_Temp_Retrieval_LowProfile_SNR50_ChainSample_2.txt',\
dtype=float)

# #Transit High 10 ppm ---------
samples10 = np.genfromtxt('PE_Transit_Retrieval_HighProfile_10ppm_ChainSample.txt',\
dtype =float)
#5 ppm
samples11 = np.genfromtxt('PE_Transit_Retrieval_HighProfile_5ppm_ChainSample.txt',\
dtype =float)
#Transit Avg 10 ppm ----------
samples13 = np.genfromtxt('PE_Transit_Retrieval_AvgProfile_10ppm_ChainSample.txt',\
dtype =float)
#5 ppm
samples14 = np.genfromtxt('PE_Transit_Retrieval_AvgProfile_5ppm_ChainSample.txt',\
dtype =float)
#Transit Low 10 ppm ----------
samples16 = np.genfromtxt('PE_Transit_Retrieval_LowProfile_10ppm_ChainSample.txt',\
dtype =float)
#5 ppm
samples17 = np.genfromtxt('PE_Transit_Retrieval_LowProfile_5ppm_ChainSample.txt',\
dtype =float)



#Colors for Histogram Plots According to each atmospheric composition
c_high = 'green'
c_avg = '#4b0082'
c_low = '#003cff'

#*~*~*~**~**~*~*~*~*~**~*~*~*~**~*~**~*~*~**~*~**~*~*~*~*~**~*~*~*~**~*~*~**~

#Subplot with essential paramters plotted in vertical orientation (Reflected)
fig, ax = plt.subplots(3,3,sharex=False,sharey=False,figsize=(10,10))

#O2 High Reflected
plt.subplot(3,3,1)
plt.hist(samples1[:,0],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_high,density=True)
plt.hist(samples2[:,0],label='30 SNR',histtype='step', alpha=0.3, \
color=c_high,linewidth=2.0,density=True)
plt.hist(samples3[:,0],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_high,density=True)
plt.axvline(np.log10(9.72e-3),linestyle='--',color='black',label='a priori')
plt.legend(loc='upper left',frameon=False)
plt.title('High',color=c_high)
plt.xlim([-10,0])

#Subpplot Labels
fig.text(0.02,0.93,'a',fontweight='bold',ha='center',fontsize=14)
fig.text(0.02,0.62,'b',fontweight='bold',ha='center',fontsize=14)
fig.text(0.02,0.30,'c',fontweight='bold',ha='center',fontsize=14)

fig.text(0.5, 0.64, 'log O$_{2}$', ha='center',fontsize=12)
fig.text(0.02, 0.5, 'Probability Density', va='center', rotation='vertical',\
          fontsize=12)

#O2 Avg Reflected
plt.subplot(3,3,2)
plt.hist(samples4[:,0],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_avg,density=True)
plt.hist(samples5[:,0],label='30 SNR',histtype='step', alpha=0.3, \
color=c_avg,linewidth=2.0,density=True)
plt.hist(samples6[:,0],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_avg,density=True)
plt.axvline(np.log10(9.69e-4),linestyle='--',color='black',label='a priori')
plt.legend(loc='upper left',frameon=False)
plt.title('Med.',color=c_avg)
plt.xlim([-10,0])
plt.ylim([0.,0.25])

#O2 Low Reflected
plt.subplot(3,3,3)
plt.hist(samples7[:,0],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_low,density=True)
plt.hist(samples8[:,0],label='30 SNR',histtype='step', alpha=0.3, \
color=c_low,linewidth=2.0,density=True)
plt.hist(samples9[:,0],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_low,density=True)
plt.axvline(np.log10(9.67e-5),linestyle='--',color='black',label='a priori')
plt.legend(loc='upper left',frameon=False)
plt.title('Low',color=c_low)
plt.xlim([-10,0])
plt.ylim([0.,0.30])


#CH4 High Reflected
plt.subplot(3,3,4)
plt.hist(samples1[:,4],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_high,density=True)
plt.hist(samples2[:,4],label='30 SNR',histtype='step', alpha=0.3, \
color=c_high,linewidth=2.0,density=True)
plt.hist(samples3[:,4],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_high,density=True)
plt.axvline(np.log10(2.85e-5),linestyle='--',color='black',label='a priori')
fig.text(0.5, 0.33, 'log CH$_{4}$', ha='center',fontsize=12)

#CH4 Avg Reflected
plt.subplot(3,3,5)
plt.hist(samples4[:,4],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_avg,density=True)
plt.hist(samples5[:,4],label='30 SNR',histtype='step', alpha=0.3, \
color=c_avg,linewidth=2.0,density=True)
plt.hist(samples6[:,4],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_avg,density=True)
plt.axvline(np.log10(1.14e-5),linestyle='--',color='black',label='a priori')

#CH4 Low Reflected
plt.subplot(3,3,6)
plt.hist(samples7[:,4],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_low,density=True)
plt.hist(samples8[:,4],label='30 SNR',histtype='step', alpha=0.3, \
color=c_low,linewidth=2.0,density=True)
plt.hist(samples9[:,4],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_low,density=True)
plt.axvline(np.log10(4.59e-6),linestyle='--',color='black',label='a priori')

#Temp High Reflected
plt.subplot(3,3,7)
plt.hist(samples1[:,6],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_high,density=True)
plt.hist(samples2[:,6],label='30 SNR',histtype='step', alpha=0.3, \
color=c_high,linewidth=2.0,density=True)
plt.hist(samples3[:,6],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_high,density=True)
fig.text(0.5, 0.02, 'T${_0}$ (K)', ha='center',fontsize=12)

#Temp Avg Reflected
plt.subplot(3,3,8)
plt.hist(samples4[:,6],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_avg,density=True)
plt.hist(samples5[:,6],label='30 SNR',histtype='step', alpha=0.3, \
color=c_avg,linewidth=2.0,density=True)
plt.hist(samples6[:,6],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_avg,density=True)

#Temp Low Reflected
plt.subplot(3,3,9)
plt.hist(samples7[:,6],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_low,density=True)
plt.hist(samples8[:,6],label='30 SNR',histtype='step', alpha=0.3, \
color=c_low,linewidth=2.0,density=True)
plt.hist(samples9[:,6],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_low,density=True)
plt.tight_layout(pad=3)
# plt.savefig('Essential_Posteriors_Reflected_2.svg')
plt.show()
plt.close()

# #*~*~*~**~**~*~*~*~*~**~*~*~*~**~*~**~*~*~**~*~**~*~*~*~*~**~*~*~*~**~*~*~**~

# #Subplot with essential paramters plotted in vertical orientation (TRANSIT)
fig, ax = plt.subplots(3,3,sharex=False,sharey=False,figsize=(10,10))

#O2 High Transit
plt.subplot(3,3,1)
plt.hist(samples10[:,0],label='10 ppm',histtype='step',alpha=0.75, \
          hatch='//',linewidth=1.0,color=c_high,density=True)
plt.hist(samples11[:,0], label='5 ppm',histtype='step',alpha=0.3,\
          color=c_high,linewidth=2.0,density=True)
fig.text(0.5, 0.65, 'log O$_{2}$', ha='center',fontsize=12)
fig.text(0.02, 0.5, 'Probability Density', va='center', rotation='vertical',\
          fontsize=12)
plt.axvline(np.log10(9.71e-3),linestyle='--',color='black',label='a priori')
plt.title('High',color=c_high)
#Subpplot Labels
fig.text(0.02,0.93,'a',fontweight='bold',ha='center',fontsize=14)
fig.text(0.02,0.62,'b',fontweight='bold',ha='center',fontsize=14)
fig.text(0.02,0.30,'c',fontweight='bold',ha='center',fontsize=14)
plt.legend(loc='upper left', frameon=False)
plt.xlim([-10,0.])
plt.ylim([0.00,0.200])

#O2 Avg Transit
plt.subplot(3,3,2)
plt.hist(samples13[:,0],label='10 ppm',histtype='step',alpha=0.75,\
          hatch='//',linewidth=1.0,color=c_avg,density=True)
plt.hist(samples14[:,0],label='5 ppm',histtype='step',alpha=0.3,\
          color=c_avg,linewidth=2.0,density=True)
plt.axvline(np.log10(9.45e-4),linestyle='--',color='black',label='a priori')
plt.legend(loc='upper left',frameon=False)
plt.title('Med.',color=c_avg)
plt.xlim([-10,0.])
plt.ylim([0.00,0.230])

#O2 Low Transit
plt.subplot(3,3,3)
plt.hist(samples16[:,0],label='10 ppm',histtype='step',alpha=0.75, \
          hatch='//',linewidth=1.0,color=c_low,density=True)
plt.hist(samples17[:,0], label='5 ppm',histtype='step',alpha=0.3,\
          color=c_low,linewidth=2.0,density=True)
plt.axvline(np.log10(9.32e-5),linestyle='--',color='black',label='a priori')

plt.title('Low',color=c_low)
plt.legend(loc='upper left',frameon=False)
plt.xlim([-10.0,0.])
plt.ylim([0.00,0.230])

#CH4 High Transit
plt.subplot(3,3,4)
plt.hist(samples10[:,4],label='10 ppm',histtype='step',alpha=0.75, \
          hatch='//',linewidth=1.0,color=c_high,density=True)
plt.hist(samples11[:,4], label='5 ppm',histtype='step',alpha=0.3,\
          color=c_high,linewidth=2.0,density=True)
plt.axvline(np.log10(7.18e-4),linestyle='--',color='black',label='a priori')
plt.xlim([-4.5,-2.0])


fig.text(0.5, 0.33, 'log CH$_{4}$', ha='center',fontsize=12)

#CH4 Avg Transit
plt.subplot(3,3,5)
plt.hist(samples13[:,4],label='10 ppm',histtype='step',alpha=0.75,\
          hatch='//',linewidth=1.0,color=c_avg,density=True)
plt.hist(samples14[:,4],label='5 ppm',histtype='step',alpha=0.3,\
          color=c_avg,linewidth=2.0,density=True)
plt.xlim([-4.5,-2.0])
plt.axvline(np.log10(2.93e-4),linestyle='--',color='black',label='a priori')

#CH4 Low Transit
plt.subplot(3,3,6)
plt.hist(samples16[:,4],label='10 ppm',histtype='step',alpha=0.75, \
          hatch='//',linewidth=1.0,color=c_low,density=True)
plt.hist(samples17[:,4], label='5 ppm',histtype='step',alpha=0.3,\
          color=c_low,linewidth=2.0,density=True)
plt.axvline(np.log10(1.19e-4),linestyle='--',color='black',label='a priori')
plt.xlim([-4.5,-2.0])

#Temp High Transit
plt.subplot(3,3,7)
plt.hist(samples10[:,6],label='10 ppm',histtype='step',alpha=0.75, \
          hatch='//',linewidth=1.0,color=c_high,density=True)
plt.hist(samples11[:,6], label='5 ppm',histtype='step',alpha=0.3,\
          color=c_high,linewidth=2.0,density=True)
plt.xlim([100.,500.])

fig.text(0.5, 0.02, 'T${_0}$ (K)', ha='center',fontsize=12)

#Temp Avg Transit
plt.subplot(3,3,8)
plt.hist(samples13[:,6],label='10 ppm',histtype='step',alpha=0.75,\
          hatch='//',linewidth=1.0,color=c_avg,density=True)
plt.hist(samples14[:,6],label='5 ppm',histtype='step',alpha=0.3,\
          color=c_avg,linewidth=2.0,density=True)
plt.xlim([100.,500.])

#Temp Low Transit
plt.subplot(3,3,9)
plt.hist(samples16[:,6],label='10 ppm',histtype='step',alpha=0.75, \
          hatch='//',linewidth=1.0,color=c_low,density=True)
plt.hist(samples17[:,6], label='5 ppm',histtype='step',alpha=0.3,\
          color=c_low,linewidth=2.0,density=True)
plt.xlim([100.,500.])


plt.tight_layout(pad=3)
# plt.savefig('Essential_Posteriors_Transit_2.svg')
plt.show()
plt.close()

# #~*~*~*~*~*~**~*~*~*~**~*~*~*~*~*~*~**~*~*~*~*~*~*~*~**~*~*~*~**~*~*~*~*~*~*~*
# #Non-essential posteriors plotted in vertical subplots for Reflected Light
fig, ax = plt.subplots(4,3,sharex=False,sharey=False,figsize=(10,10))

#H2O High Reflected
plt.subplot(4,3,1)
plt.hist(samples1[:,1],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_high,density=True)
plt.hist(samples2[:,1],label='30 SNR',histtype='step', alpha=0.3, \
color=c_high,linewidth=2.0,density=True)
plt.hist(samples3[:,1],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_high,density=True)
fig.text(0.5, 0.73, 'log H$_{2}$O', ha='center',fontsize=12)
fig.text(0.02, 0.55, 'Probability Density', va='center', rotation='vertical',\
          fontsize=12)
plt.legend(loc='upper left',frameon=False)
plt.title('High',color=c_high)
plt.xlim([-5,0])

#Subpplot Labels
fig.text(0.02,0.93,'a',fontweight='bold',ha='center',fontsize=14)
fig.text(0.02,0.70,'b',fontweight='bold',ha='center',fontsize=14)
fig.text(0.02,0.43,'c',fontweight='bold',ha='center',fontsize=14)
fig.text(0.02,0.20,'d',fontweight='bold',ha='center',fontsize=14)


#H2O Avg Reflected
plt.subplot(4,3,2)
plt.hist(samples4[:,1],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_avg,density=True)
plt.hist(samples5[:,1],label='30 SNR',histtype='step', alpha=0.3, \
color=c_avg,linewidth=2.0,density=True)
plt.hist(samples6[:,1],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_avg,density=True)
plt.legend(loc='upper left',frameon=False)
plt.title('Med.',color=c_avg)
plt.xlim([-5,0])

#H2O Low Relfected
plt.subplot(4,3,3)
plt.hist(samples7[:,1],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_low,density=True)
plt.hist(samples8[:,1],label='30 SNR',histtype='step', alpha=0.3, \
color=c_low,linewidth=2.0,density=True)
plt.hist(samples9[:,1],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_low,density=True)
plt.legend(loc='best',frameon=False)
plt.title('Low',color=c_low)
plt.xlim([-5,0])


#CO2 High Relfected
plt.subplot(4,3,4)
plt.hist(samples1[:,2],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_high,density=True)
plt.hist(samples2[:,2],label='30 SNR',histtype='step', alpha=0.3, \
color=c_high,linewidth=2.0,density=True)
plt.hist(samples3[:,2],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_high,density=True)
fig.text(0.5, 0.49, 'log CO$_{2}$', ha='center',fontsize=12)
plt.xlim([-10,0])

#CO2 Avg Reflected
plt.subplot(4,3,5)
plt.hist(samples4[:,2],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_avg,density=True)
plt.hist(samples5[:,2],label='30 SNR',histtype='step', alpha=0.3, \
color=c_avg,linewidth=2.0,density=True)
plt.hist(samples6[:,2],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_avg,density=True)
plt.xlim([-10,0])

#CO2 Low Reflected
plt.subplot(4,3,6)
plt.hist(samples7[:,2],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_low,density=True)
plt.hist(samples8[:,2],label='30 SNR',histtype='step', alpha=0.3, \
color=c_low,linewidth=2.0,density=True)
plt.hist(samples9[:,2],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_low,density=True)
plt.xlim([-10,0])

#O3 High Reflected
plt.subplot(4,3,7)
plt.hist(samples1[:,3],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_high,density=True)
plt.hist(samples2[:,3],label='30 SNR',histtype='step', alpha=0.3, \
color=c_high,linewidth=2.0,density=True)
plt.hist(samples3[:,3],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_high,density=True)
fig.text(0.5, 0.26, 'log O$_{3}$', ha='center',fontsize=12)
plt.xlim([-8,-5])

#O3 Avg Reflected
plt.subplot(4,3,8)
plt.hist(samples4[:,3],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_avg,density=True)
plt.hist(samples5[:,3],label='30 SNR',histtype='step', alpha=0.3, \
color=c_avg,linewidth=2.0,density=True)
plt.hist(samples6[:,3],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_avg,density=True)
plt.xlim([-8,-5])

#O3 Low Reflected
plt.subplot(4,3,9)
plt.hist(samples7[:,3],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_low,density=True)
plt.hist(samples8[:,3],label='30 SNR',histtype='step', alpha=0.3, \
color=c_low,linewidth=2.0,density=True)
plt.hist(samples9[:,3],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_low,density=True)
plt.xlim([-8,-5])

#P High Reflected
plt.subplot(4,3,10)
plt.hist(samples1[:,5],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_high,density=True)
plt.hist(samples2[:,5],label='30 SNR',histtype='step', alpha=0.3, \
color=c_high,linewidth=2.0,density=True)
plt.hist(samples3[:,5],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_high,density=True)
plt.xlim([3,8])
fig.text(0.5, 0.02, 'log P${_0}$ (Pa)', ha='center',fontsize=12)

#P Avg Reflected
plt.subplot(4,3,11)
plt.hist(samples4[:,5],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_avg,density=True)
plt.hist(samples5[:,5],label='30 SNR',histtype='step', alpha=0.3, \
color=c_avg,linewidth=2.0,density=True)
plt.hist(samples6[:,5],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_avg,density=True)
plt.xlim([3,8])

#P Low Reflected
plt.subplot(4,3,12)
plt.hist(samples7[:,5],label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_low,density=True)
plt.hist(samples8[:,5],label='30 SNR',histtype='step', alpha=0.3, \
color=c_low,linewidth=2.0,density=True)
plt.hist(samples9[:,5],label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_low,density=True)
plt.xlim([3,8])


plt.tight_layout(pad=3)
# plt.savefig('NonEssential_Posteriors_Reflected_2.svg')
plt.show()
plt.close()
